 /* -*- Mode: c -*-

     Lexer for a minimal programming language "nanoLang"

     Copyright 2015 by Stephan Schulz, schulz@eprover.org.

     This code is released under the GNU General Public Licence
     Version 2.
 */

%option noyywrap
%option yylineno

%{
   #include <string.h>
   #include "nanoparse.tab.h"   

   int ret_tok(int tok)
   {
      return tok;
   }

   int yycol = 0;

#define YY_USER_ACTION {yylloc.first_line = yylineno; yylloc.first_column = yycol; yycol += yyleng;}

%}

DIGIT      [0-9]
ALPHA      [a-zA-Z]
ALNUM      {ALPHA}|{DIGIT}

INT        {DIGIT}+
ID         {ALPHA}{ALNUM}*


%%

"("  {
      return ret_tok(OPENPAR);}
")"  {
      return ret_tok(CLOSEPAR);}
"*"  {
      return ret_tok(MULT);}
"/"  {
      return ret_tok(DIV);}
"+"  {
      return ret_tok(PLUS);}
"-"  {
      return ret_tok(MINUS);}
"="  {
      return ret_tok(EQ);}
"!=" {
      return ret_tok(NEQ);}
">"  {
      return ret_tok(GT);}
"<"  {
      return ret_tok(LT);}
">=" {
      return ret_tok(GEQ);}
"<=" {
      return ret_tok(LEQ);}
"{"  {
      return ret_tok(OPENCURLY);}
"}"  {
      return ret_tok(CLOSECURLY);}
";"  {
      return ret_tok(SEMICOLON);}
","  {
      return ret_tok(COMA);}


Integer {
   return ret_tok(INTEGER);}
String  {
   return ret_tok(STRING);}
if      {
   return ret_tok(IF);}
while   {
   return ret_tok(WHILE);}
return  {
   return ret_tok(RETURN);}
print   {
   return ret_tok(PRINT);}
       
{ALPHA}{ALNUM}* {return ret_tok(IDENT);}

\"[^"]*\"       {return ret_tok(STRINGLIT);}

{DIGIT}+        {return ret_tok(INTLIT);}


#[^\n]*\n { /* Eat comments */}

[\n]    {yycol = 0;}

[\r\t ] { /* Skip  whitespace*/ }


 /* Everything else is an invalid character. */
.   {return ret_tok(ERROR);}

%%

